@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
 #
 # Authors: Tim Legant <tim@catseye.net> and
 #          Jason R. Mastaler <jason@mastaler.com>
@@ -37,6 +37,7 @@ import sys
 import time
 import types
 
+import Defaults
 import Util
 
 
@@ -221,7 +222,7 @@ class FilterParser:
     bol_comment = re.compile(r'\s*#')
 
     most_sources = re.compile(r"""
-    ( (?:to|from)-(?:file|cdb|dbm|ezmlm|mailman|mysql)
+    ( (?:to|from)-(?:file|cdb|dbm|ezmlm|mailman|sql)
     | size | pipe
     | (?:to|from) (?!-) )
     """, re.VERBOSE | re.IGNORECASE)
@@ -232,26 +233,25 @@ class FilterParser:
     """, re.VERBOSE | re.IGNORECASE)
 
     matches = re.compile(r"""
-    (?: ([\'\"]) ( (?: \\\1 | [^\1] )+ ) \1
+    (?: ([\'\"]) ( (?: \\\1 | . )+? ) \1
     | ( \S+ ) )
     """, re.VERBOSE)
         
     tag_action = re.compile(r"""
     ( [A-Za-z][-\w]+ )
     \s+
-    (?: ([\'\"]) ( (?: \\\2 | [^\2] )+ ) \2
+    (?: ([\'\"]) ( (?: \\\2 | . )+? ) \2
     | ( \S+ ) )
     """, re.VERBOSE)
 
     in_action = re.compile(r"""
-    ( bounce | reject
-    | drop | exit | stop
-    | confirm | hold
-    | (?: deliver | ok | accept)(?:\s*=.*$)? )
+    ( drop | exit | stop
+    | hold
+    | (?: confirm | bounce | reject | deliver | ok | accept)(?:\s*=.*$)? )
     """, re.VERBOSE | re.IGNORECASE)
     
     out_action = re.compile(r"""
-    ( (?:(?:bare|sender|dated)(?:=\S+)?)
+    ( (?:(?:bare|sender|domain|dated)(?:=\S+)?)
     | (?:(?:exp(?:licit)?|as|ext(?:ension)?|kw|keyword)=\S+)
     | default )""", re.VERBOSE | re.IGNORECASE)
     
@@ -262,18 +262,18 @@ class FilterParser:
     arguments = {
         'from'         : None,
         'to'           : None,
-        'from-file'    : ('autocdb', 'autodbm', 'domains', 'optional'),
-        'to-file'      : ('autocdb', 'autodbm', 'domains', 'optional'),
-        'from-cdb'     : ('domains', 'optional',),
-        'to-cdb'       : ('domains', 'optional',),
-        'from-dbm'     : ('domains', 'optional',),
-        'to-dbm'       : ('domains', 'optional',),
+        'from-file'    : ('autocdb', 'autodbm', 'optional'),
+        'to-file'      : ('autocdb', 'autodbm', 'optional'),
+        'from-cdb'     : ('optional',),
+        'to-cdb'       : ('optional',),
+        'from-dbm'     : ('optional',),
+        'to-dbm'       : ('optional',),
         'from-ezmlm'   : ('optional',),
         'to-ezmlm'     : ('optional',),
         'from-mailman' : ('attr', 'optional' ),
         'to-mailman'   : ('attr', 'optional' ),
-        'from-mysql'   : ('like', 'rlike'),
-        'to-mysql'     : ('like', 'rlike'),
+        'from-sql'     : ('action_column', 'addr_column', 'wildcards'),
+        'to-sql'       : ('action_column', 'addr_column', 'wildcards'),
         'body'         : ('case',),
         'headers'      : ('case',),
         'body-file'    : ('case', 'optional'),
@@ -282,11 +282,9 @@ class FilterParser:
         'pipe'         : None
         }
 
-    # MySQL connection
-    MySQL = None
 
-
-    def __init__(self):
+    def __init__(self, db_instance=None):
+        self.db_instance = db_instance
         self.macros = []
         self.files = []
         self.filterlist = []
@@ -497,7 +495,6 @@ class FilterParser:
     def __findvarsub(self, var):
         """Look up 'var' in the Defaults namespace and the environment."""
         # First, check the Defaults namespace.
-        import Defaults
         sub = Defaults.__dict__.get(var, None)
         # Then, try the environment.
         if not sub:
@@ -687,22 +684,18 @@ class FilterParser:
 	return actions
 	
 
-    def __search_file(self, pathname, keys, actions, source):
-        """
-        Search a text file for match in first column.
-        """
-        found_match = 0
-        match_list = Util.file_to_list(pathname)
+    def __search_list(self, addrlist, keys, actions, source):
+        """Search addrlist for match in field 1, optional action in 2."""
         # This list comprehension splits each line in the list into
         # two columns, lowercases the first column and stuffs the
         # columns back together.  The result is the original list of
         # lines from the file with the first field in each line
         # lowercased.
-        match_list = [' '.join(
+        addrlist = [' '.join(
             apply(lambda f1, f2=None: f2 and [f1.lower(), f2]
                                           or [f1.lower()],
-                  line.split(None, 1))) for line in match_list]
-        found_match = Util.findmatch(match_list, keys)
+                  line.split(None, 1))) for line in addrlist]
+        found_match = Util.findmatch(addrlist, keys)
         if found_match:
             # The second column of the line may contain an
             # overriding action specification.
@@ -714,42 +707,14 @@ class FilterParser:
         return found_match
 
 
-    def __search_mysql(self, Table, Args, Keys, Actions, Source):
-        "Search MySQL table."
-        if not self.MySQL:
-            # Connect to the database if we have not yet connected
-            import Defaults, _mysql
-            self.MySQL = _mysql.connect \
-            (
-                host = Defaults.MYSQL_HOST,
-                db = Defaults.MYSQL_DATABASE,
-                user = Defaults.MYSQL_USER,
-                passwd = Defaults.MYSQL_PASSWORD
-            )
-        # Searches can be for records that are "=", "like", or "rlike" the
-        # keys.  "=" is the fastest and the default, but it does not allow any
-        # wildcarding.  "like" allows "_" to mean any character and "%" to
-        # mean any string.  "rlike" allows regular expressions.
-        Compare = "="
-        if Args.has_key('like'): Compare = "LIKE"
-        elif Args.has_key('rlike'): Compare = "RLIKE"
-        Test = ""
-        for Key in Keys:
-            if Test: Test += "OR "
-            Test += "('%s' %s ADDRESS) " % (Key, Compare)
-        # Perform the query
-        self.MySQL.query("SELECT * FROM %s WHERE %s LIMIT 1" % (Table, Test))
-        # Fetch the result
-        Result = self.MySQL.store_result().fetch_row(0, 1)
-        found_match = len(Result)
-
-        # If there is an entry for this key, we consider it an overriding
-        # action specification.
-        if found_match and Result[0]["ACTION"]:
-            Actions.clear()
-            Actions.update(self.__buildactions(Result[0]["ACTION"], Source))
-
-        return found_match
+    def __search_file(self, pathname, keys, actions, source):
+        """
+        Search a text file for match in first column.
+        """
+        return self.__search_list(Util.file_to_list(pathname),
+                                  keys,
+                                  actions,
+                                  source)
 
 
     def __search_cdb(self, pathname, keys, actions, source):
@@ -819,8 +784,7 @@ class FilterParser:
             if db_mtime <= txt_mtime:
                 if build_func(basename):
                     if os.path.exists(surrogate):
-                        mtime = time.time()
-                        os.utime(surrogate, (mtime, mtime))
+                        os.utime(surrogate, None)
                     else:
                         os.close(os.open(surrogate, os.O_CREAT, 0600))
                 else:
@@ -833,14 +797,80 @@ class FilterParser:
         """
         Attempt to extract the domain name from each address in keys.
         """
-        domains = []
+        domains = {}
         for k in keys:
             try:
-                domains.append(k.split('@', 1)[1])
+                domains[k.split('@', 1)[1]] = None
             except IndexError:
                 pass
-        return domains
-     
+        return domains.keys()
+
+
+    def __create_sql_criteria(self, dbkeys, addresscolumn):
+        """Return condition string for insertion into SQL statement."""
+        if not dbkeys:
+            return ''
+        criteria = "("
+        for i in range(len(dbkeys)):
+            if i > 0:
+                criteria += " OR "
+            criteria += "%s = %%(criterion%d)s" % (addresscolumn, i)
+        criteria += ")"
+        return criteria
+
+
+    def __get_column_index(self, colname, cursor):
+        """Return index of column named 'colname'."""
+        for i in range(len(cursor.description)):
+            if colname == cursor.description[i][0]:
+                return i
+        return -1
+
+
+    def __search_sql(self, selectstmt, args, keys, actions, source, lineno):
+        """Search SQL DB (Python DB API 2.0)."""
+        found_match = 0
+        dbkeys = keys
+        if args.has_key('wildcards'):
+            dbkeys = []
+        _username = Defaults.USERNAME.lower()
+        _hostname = Defaults.HOSTNAME.lower()
+        _recipient = _username + '@' + _hostname
+        params = create_sql_params(dbkeys,
+                                   recipient=_recipient,
+                                   username=_username,
+                                   hostname=_hostname)
+        cursor = self.db_instance.cursor()
+        try:
+            cursor.execute(selectstmt, params)
+            rows = cursor.fetchall()
+            if cursor.rowcount <= 0:
+                return 0
+            if args.has_key('wildcards'):
+                if len(cursor.description) > 1:
+                    dblist = [' '.join([row[0], row[1] or '']) for row in rows]
+                else:
+                    dblist = [row[0] for row in rows]
+                found_match = self.__search_list(dblist, keys, actions, source)
+            else:
+                action_column = args.get('action_column')
+                if action_column:
+                    actcolidx = self.__get_column_index(action_column, cursor)
+                    if actcolidx == -1:
+                        actcolidx = self.__get_column_index(
+                            action_column.lower(), cursor)
+                        if actcolidx == -1:
+                            err = "no action column (%s)" % (action_column,)
+                            raise MatchError(lineno, err)
+                    action = rows[0][actcolidx]
+                    if action:
+                        actions.clear()
+                        actions.update(self.__buildactions(action, source))
+                found_match = 1
+        finally:
+            cursor.close()
+        return found_match
+
 
     def firstmatch(self, recipient, senders=None,
                    msg_body=None, msg_headers=None, msg_size=None):
@@ -868,8 +898,7 @@ class FilterParser:
             if source in ('from-file', 'to-file'):
                 dbname = os.path.expanduser(match)
                 search_func = self.__search_file
-                if args.has_key('domains'):
-                    keys += self.__extract_domains(keys)
+                keys += self.__extract_domains(keys)
                 # If we have an 'auto*' argument, ensure that the database
                 # is up-to-date.  If the 'optional' argument is also given,
                 # don't die if the file doesn't exist.
@@ -897,8 +926,7 @@ class FilterParser:
             if source in ('from-dbm', 'to-dbm'):
                 import anydbm
                 match = os.path.expanduser(match)
-                if args.has_key('domains'):
-                    keys += self.__extract_domains(keys)
+                keys += self.__extract_domains(keys)
                 try:
                     found_match = self.__search_dbm(match, keys,
                                                     actions, source)
@@ -911,8 +939,7 @@ class FilterParser:
             if source in ('from-cdb', 'to-cdb'):
                 import cdb
                 match = os.path.expanduser(match)
-                if args.has_key('domains'):
-                    keys += self.__extract_domains(keys)
+                keys += self.__extract_domains(keys)
                 try:
                     found_match = self.__search_cdb(match, keys,
                                                     actions, source)
@@ -982,14 +1009,28 @@ class FilterParser:
                             break
                 if found_match:
 		    break
-            # MySQL-style databases.
-            if source in ('from-mysql', 'to-mysql'):
-                found_match = self.__search_mysql \
-                (
-                    match, args, keys, actions, source
-                )
+            # Generic SQL.  Expects a SELECT statement as the 'match' field.
+            # There are two "modes", depending on the presence of TMDA-style
+            # wildcards in the database.  See the filter source documentation
+            # for more information.
+            if source in ('from-sql', 'to-sql'):
+                selectstmt = match
+                keys += self.__extract_domains(keys)
+                addr_column = args.get('addr_column')
+                if args.has_key('wildcards'):
+                    if addr_column:
+                        raise MatchError(lineno,
+                                         "-addr_column and -wildcards " +
+                                         "cannot be used together")
+                elif not addr_column:
+                    raise MatchError(lineno, "-addr_column must be specified")
+                else:
+                    criteria = self.__create_sql_criteria(keys, addr_column)
+                    selectstmt = selectstmt.replace('%(criteria)s', criteria)
+                found_match = self.__search_sql(
+                    selectstmt, args, keys, actions, source, lineno)
                 if found_match:
-		    break
+                    break
             # A match is found if the command exits with a zero exit
             # status.
             if source == 'pipe' and msg_body and msg_headers:
@@ -1135,3 +1176,12 @@ def splitaction(action):
     if len(parts) == 1:
         return (parts[0], None)
     return tuple(parts)
+
+
+def create_sql_params(dbkeys=[], **kwargs):
+    """Return dictionary of parameters for SQL statement."""
+    params = kwargs.copy()
+    for i in range(len(dbkeys)):
+        params['criterion'+str(i)] = dbkeys[i]
+    return params
+
