@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
 #
 # This file is part of TMDA.
 #
@@ -25,7 +25,6 @@
 import os
 import sys
 
-import Defaults
 import Deliver
 import Errors
 import Util
@@ -33,12 +32,13 @@ import Util
 
 class MTA:
     """Non-qmail methods and instance variables. """
-    def __init__(self):
+    def __init__(self, default_delivery):
         # Exit status codes; see /usr/include/sysexits.h
         self.EX_HARD = 77               # permission denied; bounce message
         self.EX_OK = 0                  # successful termination; exit
         self.EX_STOP = None             # Non-qmail MTAs don't have such an exit code
         self.EX_TEMPFAIL = 75           # temporary failure; defer delivery
+        self.default_delivery = default_delivery
         
     # Define the four states of a message.
     def bounce(self):
@@ -48,32 +48,34 @@ class MTA:
         sys.exit(self.EX_TEMPFAIL)
 
     def stop(self):
+        if self.default_delivery == '_filter_':
+            sys.exit(99)
         sys.exit(self.EX_OK)
 
     def deliver(self, msg, instruction=None):
-        if instruction is None:
-            instruction = Defaults.DELIVERY
+        if instruction is None or self.default_delivery == '_filter_':
+            instruction = self.default_delivery
         msg = Deliver.Deliver(msg, instruction)
         msg.deliver()
-        self.stop()
-            
+        sys.exit(0)
+
 
 class Exim(MTA):
     """Exim-specific methods and instance variables."""
-    def __init__(self):
-        MTA.__init__(self)
+    def __init__(self, default_delivery):
+        MTA.__init__(self, default_delivery)
 
 
 class Postfix(MTA):
     """Postfix-specific methods and instance variables."""
-    def __init__(self):
-        MTA.__init__(self)
+    def __init__(self, default_delivery):
+        MTA.__init__(self, default_delivery)
 
 
 class Qmail(MTA):
     """qmail-specific methods and instance variables."""
-    def __init__(self):
-        MTA.__init__(self)
+    def __init__(self, default_delivery):
+        MTA.__init__(self, default_delivery)
         # qmail exit status codes; see qmail-command(8)
         self.EX_HARD = 100              # hard error; bounce message
         self.EX_OK = 0                  # success; process next instruction
@@ -91,33 +93,33 @@ class Qmail(MTA):
         sys.exit(self.EX_STOP)
 
     def deliver(self, msg, instruction=None):
-        if instruction is None:
-            instruction = Defaults.DELIVERY
+        if instruction is None or self.default_delivery == '_filter_':
+            instruction = self.default_delivery
         if instruction == '_qok_':
             sys.exit(self.EX_OK)
         else:
             msg = Deliver.Deliver(msg, instruction)
             msg.deliver()
+            if instruction == '_filter_':
+                sys.exit(self.EX_OK)
             self.stop()
 
 
 class Sendmail(MTA):
     """Sendmail-specific methods and instance variables."""
-    def __init__(self):
-        MTA.__init__(self)
+    def __init__(self, default_delivery):
+        MTA.__init__(self, default_delivery)
 
 
-def init():
-    """Factory function which determine what MTA we are running and
-    instantiates the corresponding MTA subclass."""
-    mta = Defaults.MAIL_TRANSFER_AGENT
+def init(mta, default_delivery):
+    """Factory function which instantiates the corresponding MTA subclass."""
     if mta == 'exim':
-        return Exim()
+        return Exim(default_delivery)
     elif mta == 'postfix':
-        return Postfix()
+        return Postfix(default_delivery)
     elif mta == 'qmail':
-        return Qmail()
+        return Qmail(default_delivery)
     elif mta == 'sendmail':
-        return Sendmail()
+        return Sendmail(default_delivery)
     else:
         raise Errors.ConfigError, "Unsupported MAIL_TRANSFER_AGENT: " + mta
