@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
 #
 # Author: David Guerizec <david@guerizec.net>
 #
@@ -24,14 +24,15 @@
 """Pending messages functions."""
 
 
+from email.Utils import parseaddr
+import email
 import glob
 import os
 import sys
 import time
 
-import email
-import Errors
 import Defaults
+import Errors
 import Util
 
 class Queue:
@@ -63,14 +64,13 @@ class Queue:
         self.verbose = verbose
         self.younger = younger
         self.pretend = pretend
-        self.wantedstdin = 0
 
         self.stdout = sys.stdout
 
 
     def initQueue(self):
         """Initialize the queue with the given parameters (see __init__)."""
-        self.pendingdir = os.path.join(Defaults.DATADIR, 'pending')
+        self.pendingdir = Defaults.PENDING_DIR
         if not os.path.exists(self.pendingdir):
             raise Errors.QueueError, 'Pending directory %s does not exist, exiting.' % self.pendingdir
     
@@ -78,14 +78,18 @@ class Queue:
         # via standard input.  (Since it's pointless to call it twice,
         # it's safe to remove any subsequent occurrences in the list after
         # processing.)
+        wantedstdin = 0
         for msgid in self.msgs:
             if msgid == '-':
-                self.wantedstdin = 1
+                wantedstdin = 1
                 for line in sys.stdin.readlines():
-                    self.msgs.append(line.strip())
+                    self.msgs.extend(line.strip().split())
                 self.msgs.remove('-')
+                # re-open stdin on the tty
+                sys.stdin.close()
+                sys.stdin = open('/dev/tty', 'r')
 
-        if not self.msgs and not self.wantedstdin:
+        if not self.msgs and not wantedstdin:
             cwd = os.getcwd()
             os.chdir(self.pendingdir)
             self.msgs = glob.glob('*.*.msg*')
@@ -183,6 +187,38 @@ class Queue:
                 return 0
         return 1
 
+    def cleanQueue(self, lifetime=None):
+        """Delete messages in the pending queue which exceed a certain
+        lifetime."""
+        # We can't use the Message class and delete method below
+        # because this would mean parsing the contents of every
+        # message regardless of whether we use PENDING_DELETE_APPEND
+        # or not.
+        self.older = 1
+        if lifetime is None:
+            self.threshold = Defaults.PENDING_LIFETIME
+        else:
+            self.threshold = lifetime
+        for msgid in self.msgs:
+            if not self.checkTreshold(msgid):
+                continue
+            # delete this message
+            msgfile = os.path.join(self.pendingdir, msgid)
+            if Defaults.PENDING_DELETE_APPEND:
+                try:
+                    msgobj = Util.msg_from_file(open(msgfile, 'r'))
+                except IOError:
+                    # in case of concurrent cleanups
+                    pass
+                else:
+                    rp = parseaddr(msgobj.get('return-path'))[1]
+                    Util.append_to_file(rp, Defaults.PENDING_DELETE_APPEND)
+            try:
+                os.unlink(msgfile)
+            except OSError:
+                # in case of concurrent cleanups
+                pass
+
     def checkDelivered(self, M):
         """Check if the message has already been delivered."""
         if M.wasDelivered():
@@ -241,7 +277,7 @@ class Queue:
         for msgid in self.msgs:
             self.count = self.count + 1
             try:
-                M = Message(msgid, self.command_recipient).initMessage()
+                M = Message(msgid, self.command_recipient)
             except Errors.MessageError, obj:
                 self.cPrint(obj)
                 continue
@@ -382,31 +418,26 @@ class Message:
     confirm_accept_address = None
     def __init__(self, msgid, recipient = None):
         self.msgid = msgid
-        self.msgfile = os.path.join(Defaults.DATADIR, 'pending', self.msgid)
+        self.msgfile = os.path.join(Defaults.PENDING_DIR, self.msgid)
         if not os.path.exists(self.msgfile):
             raise Errors.MessageError, '%s not found!' % self.msgid
-
-        self.msgobj = email.message_from_file(open(self.msgfile, 'r'))
+        try:
+            self.msgobj = email.message_from_file(open(self.msgfile, 'r'))
+        except email.Errors.MessageError:
+            self.msgobj = Util.msg_from_file(open(self.msgfile, 'r'))
         self.recipient = recipient
-
-    def initMessage(self, recipient = None):
-        self.return_path = email.Utils.parseaddr(
-                                        self.msgobj.get('return-path'))[1]
-        if not recipient and not self.recipient:
+        if self.recipient is None:
             self.recipient = self.msgobj.get('x-tmda-recipient')
-        else:
-            self.recipient = recipient
-        return self
+        self.return_path = parseaddr(self.msgobj.get('return-path'))[1]
+        self.x_primary_address = parseaddr(self.msgobj.get('x-primary-address'))[1]
+        self.append_address = Util.confirm_append_address(
+            self.x_primary_address, self.return_path)
 
     def release(self):
         """Release a message from the pending queue."""
-        try:
-            self.return_path
-        except:
-            self.initMessage()
         import Cookie
         if Defaults.PENDING_RELEASE_APPEND:
-            Util.append_to_file(self.return_path,
+            Util.append_to_file(self.append_address,
                                 Defaults.PENDING_RELEASE_APPEND)
         timestamp, pid, suffix = self.msgid.split('.')
         # Remove Return-Path: to avoid duplicates.
@@ -425,27 +456,19 @@ class Message:
         del self.msgobj['X-TMDA-Released']
         self.msgobj['X-TMDA-Released'] = Util.unixdate()
         # Reinject the message to the original envelope recipient.
-        Util.sendmail(self.msgobj.as_string(), self.recipient, self.return_path)
+        Util.sendmail(self.show(), self.recipient, self.return_path)
 
     def delete(self):
         """Delete a message from the pending queue."""
-        try:
-            self.return_path
-        except:
-            self.initMessage()
         if Defaults.PENDING_DELETE_APPEND:
-            Util.append_to_file(self.return_path,
+            Util.append_to_file(self.append_address,
                                 Defaults.PENDING_DELETE_APPEND)
         os.unlink(self.msgfile)
 
     def whitelist(self):
         """Whitelist the message sender."""
-        try:
-            self.return_path
-        except:
-            self.initMessage()
         if Defaults.PENDING_WHITELIST_APPEND:
-            Util.append_to_file(self.return_path,
+            Util.append_to_file(self.append_address,
                                 Defaults.PENDING_WHITELIST_APPEND)
         else:
             raise Errors.ConfigError, \
@@ -453,12 +476,8 @@ class Message:
 
     def blacklist(self):
         """Blacklist the message sender."""
-        try:
-            self.return_path
-        except:
-            self.initMessage()
         if Defaults.PENDING_BLACKLIST_APPEND:
-            Util.append_to_file(self.return_path,
+            Util.append_to_file(self.append_address,
                                 Defaults.PENDING_BLACKLIST_APPEND)
         else:
             raise Errors.ConfigError, \
@@ -470,7 +489,12 @@ class Message:
 
     def show(self):
         """Return the string representation of a message."""
-        return self.msgobj.as_string()
+        try:
+            return Util.msg_as_string(self.msgobj)
+        except TypeError:
+            # Re-parse using HeaderParser if Generator fails.
+            self.msgobj = Util.msg_from_file(open(self.msgfile, 'r'))
+            return Util.msg_as_string(self.msgobj)
 
     def getDate(self):
         timestamp = self.msgid.split('.')[0]
@@ -481,8 +505,8 @@ class Message:
         terse_hdrs = []
         for hdr in Defaults.TERSE_SUMMARY_HEADERS:
             if hdr in ('from_name', 'from_address'):
-                from_name, from_address = email.Utils.parseaddr(
-                                                self.msgobj.get('from'))
+                from_name, from_address = parseaddr(
+                    self.msgobj.get('from'))
                 if hdr == 'from_name':
                     terse_hdrs.append(from_name
                                       or from_address or 'None')
@@ -519,7 +543,7 @@ class Message:
     def summary(self, count = 0, total = 0, mailto = 0):
         """Return summary header information."""
         if not self.msg_size:
-            self.msg_size = len(self.msgobj.as_string())
+            self.msg_size = len(self.show())
             if  self.msg_size == 1:
                 self.bytes =    self.bytes[:-1]
         str = self.msgid + " ("
